@@ -1,9 +1,9 @@
 package Net::DNS::Domain;
 
-#
-# $Id: Domain.pm 1601 2017-10-10 14:17:01Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1601 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Domain.pm 1813 2020-10-08 21:58:40Z willem $)[2];
 
 
 =head1 NAME
@@ -14,7 +14,7 @@ Net::DNS::Domain - DNS domains
 
     use Net::DNS::Domain;
 
-    $domain = new Net::DNS::Domain('example.com');
+    $domain = Net::DNS::Domain->new('example.com');
     $name   = $domain->name;
 
 =head1 DESCRIPTION
@@ -35,8 +35,6 @@ partially mitigated by use of caches.
 =cut
 
 
-use strict;
-use warnings;
 use integer;
 use Carp;
 
@@ -46,14 +44,14 @@ use constant ASCII => ref eval {
 	Encode::find_encoding('ascii');
 };
 
-use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see UTR#16 3.6]
+use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see Unicode TR#16 3.6]
 	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
 };
 
-use constant LIBIDN  => defined eval 'require Net::LibIDN';
-use constant LIBIDN2 => ref eval 'require Net::LibIDN2; Net::LibIDN2->can("idn2_to_ascii_8")';
+use constant LIBIDN2 => ref eval { require Net::LibIDN2; Net::LibIDN2->can("idn2_to_ascii_8"); };
+use constant LIBIDN  => LIBIDN2 ? undef : defined eval { require Net::LibIDN };
 
-use constant IDN2FLAG => eval 'Net::LibIDN2::IDN2_NFC_INPUT + Net::LibIDN2::IDN2_NONTRANSITIONAL';
+use constant IDN2FLAG => LIBIDN2 ? &Net::LibIDN2::IDN2_NFC_INPUT + &Net::LibIDN2::IDN2_NONTRANSITIONAL : 0;
 
 # perlcc: address of encoding objects must be determined at runtime
 my $ascii = ASCII ? Encode::find_encoding('ascii') : undef;	# Osborn's Law:
@@ -64,7 +62,7 @@ my $utf8  = UTF8  ? Encode::find_encoding('utf8')  : undef;	# Variables won't; c
 
 =head2 new
 
-    $object = new Net::DNS::Domain('example.com');
+    $object = Net::DNS::Domain->new('example.com');
 
 Creates a domain object which represents the DNS domain specified
 by the character string argument. The argument consists of a
@@ -113,26 +111,24 @@ sub new {
 	my $label = $self->{label} = ( $s eq '@' ) ? [] : [split /\056/, _encode_utf8($s)];
 
 	foreach (@$label) {
+		croak 'empty domain label' unless length;
 
 		if ( LIBIDN2 && UTF8 && /[^\000-\177]/ ) {
 			my $rc = 0;
-			$_ = "\357\277\275" if /\134\d+/;	# disallow numeric escape
+			s/\134/\357\277\275/;			# disallow escapes
 			$_ = Net::LibIDN2::idn2_to_ascii_8( $_, IDN2FLAG, $rc );
 			croak Net::LibIDN2::idn2_strerror($rc) unless $_;
 		}
 
-		if ( !LIBIDN2 && LIBIDN && UTF8 && /[^\000-\177]/ ) {
-			$_ = "\357\277\275" if /\134\d+/;	# disallow numeric escape
+		if ( LIBIDN && UTF8 && /[^\000-\177]/ ) {
+			s/\134/\357\277\275/;			# disallow escapes
 			$_ = Net::LibIDN::idn_to_ascii( $_, 'utf-8' );
-			croak 'invalid name' unless $_;
+			croak 'name contains disallowed character' unless $_;
 		}
 
 		s/\134([\060-\071]{3})/$unescape{$1}/eg;	# numeric escape
 		s/\134(.)/$1/g;					# character escape
-		croak 'empty domain label' unless length;
-		next unless length > 63;
-		substr( $_, 63 ) = '';
-		carp 'domain label truncated';
+		croak 'long domain label' if length > 63;
 	}
 
 	$$cache1{$k} = $self;					# cache object reference
@@ -164,10 +160,13 @@ sub name {
 	return $self->{name} if defined $self->{name};
 	return unless defined wantarray;
 
-	my @label = map { s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg; $_ } $self->_wire;
+	my @label = shift->_wire;
+	for (@label) {
+		s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg;
+	}
 
 	return $self->{name} = '.' unless scalar @label;
-	$self->{name} = _decode_ascii( join chr(46), @label );
+	return $self->{name} = _decode_ascii( join chr(46), @label );
 }
 
 
@@ -207,7 +206,7 @@ sub xname {
 		return $self->{xname} = $u8 ? $utf8->decode($u8) : $name;
 	}
 
-	if ( !LIBIDN2 && LIBIDN && UTF8 && $name =~ /xn--/i ) {
+	if ( LIBIDN && UTF8 && $name =~ /xn--/i ) {
 		my $self = shift;
 		return $self->{xname} if defined $self->{xname};
 		return $self->{xname} = $utf8->decode( Net::LibIDN::idn_to_unicode $name, 'utf-8' );
@@ -225,19 +224,12 @@ Identifies the domain by means of a list of domain labels.
 =cut
 
 sub label {
-	map {
+	my @label = shift->_wire;
+	for (@label) {
 		s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg;
 		_decode_ascii($_)
-	} shift->_wire;
-}
-
-
-sub _wire {
-	my $self = shift;
-
-	my $label = $self->{label};
-	my $origin = $self->{origin} || return (@$label);
-	return ( @$label, $origin->_wire );
+	}
+	return @label;
 }
 
 
@@ -254,16 +246,16 @@ represented by the appropriate escape sequence.
 =cut
 
 sub string {
-	( my $name = &name ) =~ s/(["'\$();@])/\\$1/;		# escape special char
+	( my $name = &name ) =~ s/(["();@])/\\$1/;		# escape special char
 	return $name =~ /[.]$/ ? $name : $name . '.';		# append trailing dot
 }
 
 
 =head2 origin
 
-    $create = origin Net::DNS::Domain( $ORIGIN );
-    $result = &$create( sub{ new Net::DNS::RR( 'mx MX 10 a' ); } );
-    $expect = new Net::DNS::RR( "mx.$ORIGIN. MX 10 a.$ORIGIN." );
+    $create = Net::DNS::Domain->origin( $ORIGIN );
+    $result = &$create( sub{ Net::DNS::RR->new( 'mx MX 10 a' ); } );
+    $expect = Net::DNS::RR->new( "mx.$ORIGIN. MX 10 a.$ORIGIN." );
 
 Class method which returns a reference to a subroutine wrapper
 which executes a given constructor in a dynamically scoped context
@@ -275,7 +267,7 @@ my $placebo = sub { my $constructor = shift; &$constructor; };
 
 sub origin {
 	my ( $class, $name ) = @_;
-	my $domain = defined $name ? new Net::DNS::Domain($name) : return $placebo;
+	my $domain = defined $name ? Net::DNS::Domain->new($name) : return $placebo;
 
 	return sub {						# closure w.r.t. $domain
 		my $constructor = shift;
@@ -296,11 +288,11 @@ sub _decode_ascii {			## ASCII to perl internal encoding
 	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
 	my $z = length($_) - length($_);			# pre-5.18 taint workaround
-	ASCII ? substr( $ascii->decode($_), $z ) : $_;
+	return ASCII ? substr( $ascii->decode($_), $z ) : $_;
 }
 
 
-sub _encode_utf8 {			## perl internal encoding to ASCII
+sub _encode_utf8 {			## perl internal encoding to UTF8
 	local $_ = shift;
 
 	# partial transliteration for non-ASCII character encodings
@@ -309,26 +301,29 @@ sub _encode_utf8 {			## perl internal encoding to ASCII
 	[\040-\176\077] unless ASCII;
 
 	my $z = length($_) - length($_);			# pre-5.18 taint workaround
-	ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->encode($_), $z ) : $_;
+	return ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->encode($_), $z ) : $_;
+}
+
+
+sub _wire {
+	my $self = shift;
+
+	my $label = $self->{label};
+	my $origin = $self->{origin};
+	return ( @$label, $origin ? $origin->_wire : () );
 }
 
 
 %escape = eval {			## precalculated ASCII escape table
-	my %table;
-
-	foreach ( 33 .. 126 ) {					# ASCII printable
-		$table{pack( 'C', $_ )} = pack 'C', $_;
-	}
-
-	# minimal character escapes
-	foreach ( 46, 92 ) {					# \. \\
-		$table{pack( 'C', $_ )} = pack 'C*', 92, $_;
-	}
+	my %table = (						# ASCII printable,	\. \\
+		( map { ( $_ => $_ ) } map { pack( 'C', $_ ) } ( 33 .. 126 ) ),
+		( map { pack( 'C', $_ ) => pack( 'C2', 92, $_ ) } ( 46, 92 ) ),
+		);
 
 	foreach my $n ( 0 .. 32, 127 .. 255 ) {			# \ddd
 		my $codepoint = sprintf( '%03u', $n );
 
-		# partial transliteration for non-ASCII character encodings
+		# transliteration for non-ASCII character encodings
 		$codepoint =~ tr [0-9] [\060-\071];
 
 		$table{pack( 'C', $n )} = pack 'C a3', 92, $codepoint;
@@ -344,12 +339,12 @@ sub _encode_utf8 {			## perl internal encoding to ASCII
 	foreach my $n ( 0 .. 255 ) {
 		my $key = sprintf( '%03u', $n );
 
-		# partial transliteration for non-ASCII character encodings
+		# transliteration for non-ASCII character encodings
 		$key =~ tr [0-9] [\060-\071];
 
 		$table{$key} = pack 'C', $n;
-		$table{$key} = pack 'C2', 92, $n if $n == 92;	   # escaped escape
 	}
+	$table{"\060\071\062"} = pack 'C2', 92, 92;		# escaped escape
 
 	return %table;
 };
@@ -399,8 +394,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::LibIDN>, L<Net::DNS>, RFC1034, RFC1035, RFC5891,
-Unicode Technical Report #16
+L<perl>, L<Net::DNS>, L<Net::LibIDN2>, RFC1034, RFC1035, RFC5891, Unicode TR#16
 
 =cut
 
