@@ -1,46 +1,44 @@
 package Net::DNS::RR::RRSIG;
 
-#
-# $Id: RRSIG.pm 1752 2019-07-23 14:41:27Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1752 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: RRSIG.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::RRSIG - DNS RRSIG resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
-use MIME::Base64;
 use Time::Local;
 
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:type);
 
 use constant DEBUG => 0;
 
-use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
+use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';	## no critic
+
+eval { require MIME::Base64 };
 
 # IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
 use constant USESEC => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
 								# Discourage static code analysers and casual greppers
-use constant DNSSEC => USESEC && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);
+use constant DNSSEC => USESEC && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);	## no critic
 
 my @index;
 if (DNSSEC) {
-	my $key = new Net::DNS::RR( type => 'DNSKEY', key => 'AwEAAQ==' );
-	my @arg = ( '', $key, '' );				# Grotesquely inefficient; but API not changing anytime soon
-	foreach my $class ( map "Net::DNS::SEC::$_", qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
-		push @index, map eval { $key->algorithm($_); $class->verify(@arg); ( $_ => $class ) }, ( 1 .. 25 )
-				if eval join '', qw(r e q u i r e), " $class";
+	my $key = Net::DNS::RR->new( type => 'DNSKEY', key => 'AwEAAQ==' );
+	foreach my $class ( map {"Net::DNS::SEC::$_"} qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
+		my @algorithms = eval join '', qw(r e q u i r e), " $class; $class->_index";	## no critic
+		@algorithms = grep { eval { $key->algorithm($_); $class->verify( '', $key, '' ); 1 } } ( 1 .. 16 )
+				unless scalar(@algorithms);	# Grotesquely inefficient; but need to support pre-1.14 API
+		push @index, map { ( $_ => $class ) } @algorithms;
 	}
 }
 
@@ -61,6 +59,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	@{$self}{@field} = unpack "\@$offset n C2 N3 n", $$data;
 	( $self->{signame}, $offset ) = decode Net::DNS::DomainName( $data, $offset + 18 );
 	$self->{sigbin} = substr $$data, $offset, $limit - $offset;
+	return;
 }
 
 
@@ -68,7 +67,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $signame = $self->{signame};
-	pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->canonical, $self->sigbin;
+	return pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->canonical, $self->sigbin;
 }
 
 
@@ -76,8 +75,9 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my $signame = $self->{signame};
-	my @sig64   = split /\s+/, encode_base64( $self->sigbin );
-	my @rdata   = ( map( $self->$_, @field ), $signame->string, @sig64 );
+	my @sig64   = split /\s+/, MIME::Base64::encode( $self->sigbin );
+	my @rdata   = ( map( { $self->$_ } @field ), $signame->string, @sig64 );
+	return @rdata;
 }
 
 
@@ -86,6 +86,7 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 
 	foreach ( @field, qw(signame) ) { $self->$_(shift) }
 	$self->signature(@_);
+	return;
 }
 
 
@@ -93,6 +94,7 @@ sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->sigval(30);
+	return;
 }
 
 
@@ -127,7 +129,8 @@ sub _defaults {				## specify RR attribute default values
 
 	my %algbyval = reverse @algbyname;
 
-	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
+	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
 	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
@@ -141,7 +144,7 @@ sub _defaults {				## specify RR attribute default values
 
 	sub _algbyval {
 		my $value = shift;
-		$algbyval{$value} || return $value;
+		return $algbyval{$value} || return $value;
 	}
 }
 
@@ -150,7 +153,7 @@ sub typecovered {
 	my $self = shift;
 	$self->{typecovered} = typebyname(shift) if scalar @_;
 	my $typecode = $self->{typecovered};
-	typebyval($typecode) if defined wantarray && defined $typecode;
+	return defined $typecode ? typebyval($typecode) : undef;
 }
 
 
@@ -172,7 +175,7 @@ sub labels {
 	my $self = shift;
 
 	$self->{labels} = 0 + shift if scalar @_;
-	$self->{labels} || 0;
+	return $self->{labels} || 0;
 }
 
 
@@ -180,7 +183,7 @@ sub orgttl {
 	my $self = shift;
 
 	$self->{orgttl} = 0 + shift if scalar @_;
-	$self->{orgttl} || 0;
+	return $self->{orgttl} || 0;
 }
 
 
@@ -200,14 +203,14 @@ sub siginception {
 	return UTIL ? Scalar::Util::dualvar( $time, _time2string($time) ) : _time2string($time);
 }
 
-sub sigex { &sigexpiration; }		## historical
+sub sigex { return &sigexpiration; }	## historical
 
-sub sigin { &siginception; }		## historical
+sub sigin { return &siginception; }	## historical
 
 sub sigval {
 	my $self = shift;
 	no integer;
-	( $self->{sigval} ) = map int( 86400 * $_ ), @_;
+	return ( $self->{sigval} ) = map { int( 86400 * $_ ) } @_;
 }
 
 
@@ -215,22 +218,22 @@ sub keytag {
 	my $self = shift;
 
 	$self->{keytag} = 0 + shift if scalar @_;
-	$self->{keytag} || 0;
+	return $self->{keytag} || 0;
 }
 
 
 sub signame {
 	my $self = shift;
 
-	$self->{signame} = new Net::DNS::DomainName(shift) if scalar @_;
-	$self->{signame}->name if $self->{signame};
+	$self->{signame} = Net::DNS::DomainName->new(shift) if scalar @_;
+	return $self->{signame} ? $self->{signame}->name : undef;
 }
 
 
 sub sig {
 	my $self = shift;
 	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @_;
-	$self->sigbin( MIME::Base64::decode( join "", @_ ) );
+	return $self->sigbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -238,11 +241,11 @@ sub sigbin {
 	my $self = shift;
 
 	$self->{sigbin} = shift if scalar @_;
-	$self->{sigbin} || "";
+	return $self->{sigbin} || "";
 }
 
 
-sub signature { &sig; }
+sub signature { return &sig; }
 
 
 sub create {
@@ -257,14 +260,14 @@ sub create {
 
 		# All the TTLs need to be the same in the data RRset.
 		my $ttl = $RR->ttl;
-		croak 'RRs in RRset do not have same TTL' if grep $_->ttl != $ttl, @$rrsetref;
+		croak 'RRs in RRset do not have same TTL' if grep { $_->ttl != $ttl } @$rrsetref;
 
 		my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
 		croak 'unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
 
-		my @label = grep $_ ne chr(42), $RR->{owner}->_wire;	# count labels
+		my @label = grep { $_ ne chr(42) } $RR->{owner}->_wire;	   # count labels
 
-		my $self = new Net::DNS::RR(
+		my $self = Net::DNS::RR->new(
 			name	     => $RR->name,
 			type	     => 'RRSIG',
 			class	     => 'IN',
@@ -285,7 +288,8 @@ sub create {
 		$self->{sigexpiration} = $self->{siginception} + $self->{sigval}
 				unless $self->{sigexpiration};
 
-		$self->_CreateSig( $self->_CreateSigData($rrsetref), $private );
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_CreateSig( $sigdata, $private );
 		return $self;
 	}
 }
@@ -359,7 +363,8 @@ sub verify {
 			return 0;
 		}
 
-		$self->_VerifySig( $self->_CreateSigData($rrsetref), $keyref ) || return 0;
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_VerifySig( $sigdata, $keyref ) || return 0;
 
 		# time to do some time checking.
 		my $t = time;
@@ -379,26 +384,26 @@ sub verify {
 
 sub vrfyerrstr {
 	my $self = shift;
-	$self->{vrfyerrstr};
+	return $self->{vrfyerrstr};
 }
 
 
 ########################################
 
-sub _ordered($$) {			## irreflexive 32-bit partial ordering
+sub _ordered() {			## irreflexive 32-bit partial ordering
 	use integer;
-	my ( $a, $b ) = @_;
+	my ( $l, $h ) = @_;
 
-	return defined $b unless defined $a;			# ( undef, any )
-	return 0 unless defined $b;				# ( any, undef )
+	return defined $h unless defined $l;			# ( undef, any )
+	return 0	  unless defined $h;			# ( any, undef )
 
 	# unwise to assume 64-bit arithmetic, or that 32-bit integer overflow goes unpunished
-	if ( $a < 0 ) {						# translate $a<0 region
-		$a = ( $a ^ 0x80000000 ) & 0xFFFFFFFF;		#  0	 <= $a < 2**31
-		$b = ( $b ^ 0x80000000 ) & 0xFFFFFFFF;		# -2**31 <= $b < 2**32
+	if ( $l < 0 ) {						# translate $l<0 region
+		$l = ( $l ^ 0x80000000 ) & 0xFFFFFFFF;		#  0	 <= $l < 2**31
+		$h = ( $h ^ 0x80000000 ) & 0xFFFFFFFF;		# -2**31 <= $h < 2**32
 	}
 
-	return $a < $b ? ( $a > ( $b - 0x80000000 ) ) : ( $b < ( $a - 0x80000000 ) );
+	return $l < $h ? ( $l > ( $h - 0x80000000 ) ) : ( $h < ( $l - 0x80000000 ) );
 }
 
 
@@ -481,7 +486,7 @@ sub _CreateSigData {
 		my $suffix = $wild->canonical;
 		unshift @label, chr(42);			# asterisk
 
-		my @RR	  = map bless( {%$_}, ref($_) ), @$rrsetref;   # shallow RR clone
+		my @RR	  = map { bless( {%$_}, ref($_) ) } @$rrsetref;	   # shallow RR clone
 		my $rr	  = $RR[0];
 		my $class = $rr->class;
 		my $type  = $rr->type;
@@ -508,7 +513,7 @@ sub _CreateSigData {
 			$table{substr $canonical, $offset} = $canonical;
 		}
 
-		$sigdata = join '', $sigdata, map $table{$_}, sort keys %table;
+		$sigdata = join '', $sigdata, map { $table{$_} } sort keys %table;
 
 		if (DEBUG) {
 			my $i = 0;
@@ -533,10 +538,10 @@ sub _CreateSig {
 		my $algorithm = $self->algorithm;
 		my $class     = $DNSSEC_siggen{$algorithm};
 
-		eval {
-			return $self->sigbin( $class->sign(@_) ) if $class;
-			die "algorithm $algorithm not supported\n";
-		} || croak "${@}signature generation failed";
+		return eval {
+			die "algorithm $algorithm not supported\n" unless $class;
+			$self->sigbin( $class->sign(@_) );
+		} || return croak "${@}signature generation failed";
 	}
 }
 
@@ -549,8 +554,8 @@ sub _VerifySig {
 		my $class     = $DNSSEC_verify{$algorithm};
 
 		my $retval = eval {
-			return $class->verify( @_, $self->sigbin ) if $class;
-			die "algorithm $algorithm not supported\n";
+			die "algorithm $algorithm not supported\n" unless $class;
+			$class->verify( @_, $self->sigbin );
 		};
 
 		unless ($retval) {
@@ -574,12 +579,12 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name RRSIG typecovered algorithm labels
+    $rr = Net::DNS::RR->new('name RRSIG typecovered algorithm labels
 				orgttl sigexpiration siginception
 				keytag signame signature');
 
     use Net::DNS::SEC;
-    $sigrr = create Net::DNS::RR::RRSIG( \@rrset, $keypath,
+    $sigrr = Net::DNS::RR::RRSIG->create( \@rrset, $keypath,
 					sigex => 20191231010101
 					sigin => 20191201010101
 					);
@@ -702,9 +707,9 @@ Create a signature over a RR set.
 
     $keypath = '/home/olaf/keys/Kbla.foo.+001+60114.private';
 
-    $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath );
+    $sigrr = Net::DNS::RR::RRSIG->create( \@rrsetref, $keypath );
 
-    $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath,
+    $sigrr = Net::DNS::RR::RRSIG->create( \@rrsetref, $keypath,
 					sigex => 20191231010101
 					sigin => 20191201010101
 					);
@@ -715,7 +720,7 @@ Create a signature over a RR set.
 
     $private = Net::DNS::SEC::Private->new($keypath);
 
-    $sigrr= create Net::DNS::RR::RRSIG( \@rrsetref, $private );
+    $sigrr= Net::DNS::RR::RRSIG->create( \@rrsetref, $private );
 
 
 create() is an alternative constructor for a RRSIG RR object.  
@@ -848,11 +853,7 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
-RFC4034, RFC6840, RFC3755,
-L<Net::DNS::SEC::DSA>,
-L<Net::DNS::SEC::ECDSA>,
-L<Net::DNS::SEC::EdDSA>,
-L<Net::DNS::SEC::RSA>
+RFC4034, RFC6840, RFC3755
 
 L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
 
