@@ -1,34 +1,30 @@
 package Net::DNS::RR::TKEY;
 
-#
-# $Id: TKEY.pm 1348 2015-05-27 09:55:13Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1348 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: TKEY.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::TKEY - DNS TKEY resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:class :type);
 use Net::DNS::DomainName;
 
 use constant ANY  => classbyname qw(ANY);
 use constant TKEY => typebyname qw(TKEY);
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -48,10 +44,11 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 	$offset += $other_size + 2;
 
 	croak('corrupt TKEY data') unless $offset == $limit;	# more or less FUBAR
+	return;
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	return '' unless defined $self->{algorithm};
@@ -68,13 +65,6 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
-
-	croak 'zone file representation not defined for TKEY' if shift;
-}
-
-
 sub class {				## overide RR method
 	return 'ANY';
 }
@@ -83,7 +73,7 @@ sub encode {				## overide RR method
 	my $self = shift;
 
 	my $owner = $self->{owner}->encode();
-	my $rdata = eval { $self->encode_rdata() } || '';
+	my $rdata = eval { $self->_encode_rdata() } || '';
 	return pack 'a* n2 N n a*', $owner, TKEY, ANY, 0, length $rdata, $rdata;
 }
 
@@ -91,8 +81,8 @@ sub encode {				## overide RR method
 sub algorithm {
 	my $self = shift;
 
-	$self->{algorithm} = new Net::DNS::DomainName(shift) if scalar @_;
-	$self->{algorithm}->name if defined wantarray && $self->{algorithm};
+	$self->{algorithm} = Net::DNS::DomainName->new(shift) if scalar @_;
+	return $self->{algorithm} ? $self->{algorithm}->name : undef;
 }
 
 
@@ -132,7 +122,7 @@ sub key {
 	my $self = shift;
 
 	$self->{key} = shift if scalar @_;
-	$self->{key} || "";
+	return $self->{key} || "";
 }
 
 
@@ -140,11 +130,11 @@ sub other {
 	my $self = shift;
 
 	$self->{other} = shift if scalar @_;
-	$self->{other} || "";
+	return $self->{other} || "";
 }
 
 
-sub other_data { &other; }		## historical
+sub other_data { return &other; }				# uncoverable pod
 
 
 1;
