@@ -1,10 +1,9 @@
 package Net::DNS::Update;
 
-#
-# $Id: Update.pm 1418 2015-10-20 09:55:07Z willem $
-#
-use vars qw($VERSION @ISA);
-$VERSION = (qw$LastChangedRevision: 1418 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Update.pm 1813 2020-10-08 21:58:40Z willem $)[2];
 
 
 =head1 NAME
@@ -15,10 +14,10 @@ Net::DNS::Update - DNS dynamic update packet
 
     use Net::DNS;
 
-    $update = new Net::DNS::Update( 'example.com', 'IN' );
+    $update = Net::DNS::Update->new( 'example.com', 'IN' );
 
-    $update->push( prereq => nxrrset('foo.example.com. A') );
-    $update->push( update => rr_add('foo.example.com. 86400 A 192.168.1.2') );
+    $update->push( prereq => nxrrset('foo.example.com. AAAA') );
+    $update->push( update => rr_add('foo.example.com. 86400 AAAA 2001::DB8::1') );
 
 =head1 DESCRIPTION
 
@@ -30,27 +29,29 @@ Programmers should refer to RFC2136 for dynamic update semantics.
 =cut
 
 
-use strict;
-use base 'Net::DNS::Packet';
-
+use integer;
 use Carp;
 
+use base qw(Net::DNS::Packet);
+
+use Net::DNS::Resolver;
+
 
 =head1 METHODS
 
 =head2 new
 
-    $update = new Net::DNS::Update;
-    $update = new Net::DNS::Update( 'example.com' );
-    $update = new Net::DNS::Update( 'example.com', 'HS' );
+    $update = Net::DNS::Update->new;
+    $update = Net::DNS::Update->new( 'example.com' );
+    $update = Net::DNS::Update->new( 'example.com', 'HS' );
 
 Returns a Net::DNS::Update object suitable for performing a DNS
 dynamic update.	 Specifically, it creates a packet with the header
 opcode set to UPDATE and the zone record type to SOA (per RFC 2136,
 Section 2.3).
 
-Programs must use the push() method to add RRs to the prerequisite,
-update, and additional sections before performing the update.
+Programs must use the push() method to add RRs to the prerequisite
+and update sections before performing the update.
 
 Arguments are the zone name and the class.  The zone and class may
 be undefined or omitted and default to the default domain from the
@@ -59,26 +60,18 @@ resolver configuration and IN respectively.
 =cut
 
 sub new {
-	shift;
-	my ( $zone, @class ) = @_;
+	my ( $class, $zone, @rrclass ) = @_;
 
-	unless ( defined $zone ) {
-		require Net::DNS::Resolver;
-		($zone) = new Net::DNS::Resolver()->domain;	# default from resolver config
-	}
+	my ($domain) = grep { defined && length } ( $zone, Net::DNS::Resolver->searchlist );
 
-	eval {
-		local $SIG{__DIE__};
+	my $self = __PACKAGE__->SUPER::new( $domain, 'SOA', @rrclass );
 
-		my $self = __PACKAGE__->SUPER::new( $zone, 'SOA', @class );
+	my $header = $self->header;
+	$header->opcode('UPDATE');
+	$header->qr(0);
+	$header->rd(0);
 
-		my $header = $self->header;
-		$header->opcode('UPDATE');
-		$header->qr(0);
-		$header->rd(0);
-
-		return $self;
-	} || croak $@;
+	return $self;
 }
 
 
@@ -102,13 +95,13 @@ Section names may be abbreviated to the first three characters.
 sub push {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	my @arg	 = grep ref($_), @_;
+	my @arg	 = grep { ref($_) } @_;
 
 	my ($zone) = $self->zone;
 	my $zclass = $zone->zclass;
-	my @rr = grep $_->class( $_->class =~ /ANY|NONE/ ? () : $zclass ), @arg;
+	my @rr = grep { $_->class( $_->class =~ /ANY|NONE/ ? () : $zclass ) } @arg;
 
-	CORE::push( @$list, @rr );
+	return CORE::push( @$list, @rr );
 }
 
 
@@ -133,15 +126,15 @@ Section names may be abbreviated to the first three characters.
 sub unique_push {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	my @arg	 = grep ref($_), @_;
+	my @arg	 = grep { ref($_) } @_;
 
 	my ($zone) = $self->zone;
 	my $zclass = $zone->zclass;
-	my @rr = grep $_->class( $_->class =~ /ANY|NONE/ ? () : $zclass ), @arg;
+	my @rr = grep { $_->class( $_->class =~ /ANY|NONE/ ? () : $zclass ) } @arg;
 
-	my %unique = map { ( bless( {%$_, ttl => 0}, ref $_ )->canonical => $_ ) } @rr, @$list;
+	my %unique = map { ( bless( {%$_, ttl => 0}, ref $_ )->canonical => $_ ) } ( @rr, @$list );
 
-	scalar( @$list = values %unique );
+	return scalar( @$list = values %unique );
 }
 
 
@@ -165,17 +158,18 @@ the corresponding ( name => value ) form may also be used.
     use Net::DNS;
 
     # Create the update packet.
-    my $update = new Net::DNS::Update('example.com');
+    my $update = Net::DNS::Update->new('example.com');
 
-    # Prerequisite is that no A records exist for the name.
+    # Prerequisite is that no address records exist for the name.
     $update->push( pre => nxrrset('foo.example.com. A') );
+    $update->push( pre => nxrrset('foo.example.com. AAAA') );
 
-    # Add two A records for the name.
-    $update->push( update => rr_add('foo.example.com. 86400 A 192.168.1.2') );
-    $update->push( update => rr_add('foo.example.com. 86400 A 172.16.3.4') );
+    # Add two address records for the name.
+    $update->push( update => rr_add('foo.example.com. 86400 A 192.0.2.1') );
+    $update->push( update => rr_add('foo.example.com. 86400 AAAA 2001:DB8::1') );
 
     # Send the update to the zone's primary master.
-    my $resolver = new Net::DNS::Resolver;
+    my $resolver = Net::DNS::Resolver->new();
     $resolver->nameservers('primary-master.example.com');
 
     my $reply = $resolver->send($update);
@@ -194,32 +188,32 @@ the corresponding ( name => value ) form may also be used.
 
 =head2 Add an MX record for a name that already exists
 
-    my $update = new Net::DNS::Update('example.com');
+    my $update = Net::DNS::Update->new('example.com');
     $update->push( prereq => yxdomain('example.com') );
     $update->push( update => rr_add('example.com MX 10 mailhost.example.com') );
 
 =head2 Add a TXT record for a name that does not exist
 
-    my $update = new Net::DNS::Update('example.com');
+    my $update = Net::DNS::Update->new('example.com');
     $update->push( prereq => nxdomain('info.example.com') );
     $update->push( update => rr_add('info.example.com TXT "yabba dabba doo"') );
 
 =head2 Delete all A records for a name
 
-    my $update = new Net::DNS::Update('example.com');
+    my $update = Net::DNS::Update->new('example.com');
     $update->push( prereq => yxrrset('foo.example.com A') );
     $update->push( update => rr_del('foo.example.com A') );
 
 =head2 Delete all RRs for a name
 
-    my $update = new Net::DNS::Update('example.com');
+    my $update = Net::DNS::Update->new('example.com');
     $update->push( prereq => yxdomain('byebye.example.com') );
     $update->push( update => rr_del('byebye.example.com') );
 
 =head2 Perform a DNS update signed using a BIND private key file
 
-    my $update = new Net::DNS::Update('example.com');
-    $update->push( update => rr_add('foo.example.com A 10.1.2.3') );
+    my $update = Net::DNS::Update->new('example.com');
+    $update->push( update => rr_add('foo.example.com AAAA 2001:DB8::1') );
     $update->sign_tsig( "$dir/Khmac-sha512.example.com.+165+01018.private" );
     my $reply = $resolver->send( $update );
     $reply->verify( $update ) || die $reply->verifyerr;
@@ -236,14 +230,13 @@ the corresponding ( name => value ) form may also be used.
 
 =head2 Another way to sign a DNS update
 
-    my $key_name = 'tsig-key';
-    my $key	 = 'awwLOtRfpGE+rRKF2+DEiw==';
+    use Net::DNS::RR::TSIG;
 
-    my $tsig = new Net::DNS::RR("$key_name TSIG $key");
+    my $tsig = create Net::DNS::RR::TSIG( $key_file );
     $tsig->fudge(60);
 
-    my $update = new Net::DNS::Update('example.com');
-    $update->push( update     => rr_add('foo.example.com A 10.1.2.3') );
+    my $update = Net::DNS::Update->new('example.com');
+    $update->push( update     => rr_add('foo.example.com AAAA 2001:DB8::1') );
     $update->push( additional => $tsig );
 
 
